@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "0.5.4"
+#define BARESIP_VERSION "0.5.5"
 
 
 #ifndef NET_MAX_NS
@@ -47,6 +47,7 @@ struct account;
 
 int account_alloc(struct account **accp, const char *sipaddr);
 int account_debug(struct re_printf *pf, const struct account *acc);
+int account_set_auth_pass(struct account *acc, const char *pass);
 int account_set_display_name(struct account *acc, const char *dname);
 int account_auth(const struct account *acc, char **username, char **password,
 		 const char *realm);
@@ -57,7 +58,9 @@ uint32_t account_regint(const struct account *acc);
 uint32_t account_pubint(const struct account *acc);
 uint32_t account_ptime(const struct account *acc);
 enum answermode account_answermode(const struct account *acc);
+const char *account_aor(const struct account *acc);
 const char *account_auth_user(const struct account *acc);
+const char *account_auth_pass(const struct account *acc);
 const char *account_outbound(const struct account *acc, unsigned ix);
 const char *account_stun_user(const struct account *acc);
 const char *account_stun_pass(const struct account *acc);
@@ -213,6 +216,7 @@ struct config_video {
 	unsigned width, height; /**< Video resolution               */
 	uint32_t bitrate;       /**< Encoder bitrate in [bit/s]     */
 	uint32_t fps;           /**< Video framerate                */
+	bool fullscreen;        /**< Enable fullscreen display      */
 };
 #endif
 
@@ -283,18 +287,28 @@ enum presence_status {
 	PRESENCE_BUSY
 };
 
+
+struct contact;
+typedef void (contact_update_h)(struct contact *c, bool removed, void *arg);
+
 struct contacts {
 	struct list cl;
 	struct hash *cht;
+
+	contact_update_h *handler;
+	void* handler_arg;
 };
 
-struct contact;
 
 int  contact_init(struct contacts *contacts);
 void contact_close(struct contacts *contacts);
 int  contact_add(struct contacts *contacts,
 		 struct contact **contactp, const struct pl *addr);
+void contact_remove(struct contacts *contacts, struct contact *c);
+void contact_set_update_handler(struct contacts *contacs,
+				contact_update_h *updateh, void *arg);
 int  contacts_print(struct re_printf *pf, const struct contacts *contacts);
+enum presence_status contact_presence(const struct contact *c);
 void contact_set_presence(struct contact *c, enum presence_status status);
 bool contact_block_access(const struct contacts *contacts, const char *uri);
 struct contact  *contact_find(const struct contacts *contacts,
@@ -642,6 +656,11 @@ struct sipevent_sock *uag_sipevent_sock(void);
  * User Interface
  */
 
+struct ui_sub {
+	struct list uil;  /**< List of UIs (struct ui) */
+	struct cmd_ctx *uictx;
+};
+
 typedef int  (ui_output_h)(const char *str);
 
 /** Defines a User-Interface module */
@@ -651,16 +670,15 @@ struct ui {
 	ui_output_h *outputh;  /**< Handler for output strings (optional) */
 };
 
-void ui_register(struct ui *ui);
+void ui_register(struct ui_sub *uis, struct ui *ui);
 void ui_unregister(struct ui *ui);
 
-void ui_reset(void);
-void ui_input(char key);
-void ui_input_key(char key, struct re_printf *pf);
+void ui_reset(struct ui_sub *uis);
+void ui_input_key(struct ui_sub *uis, char key, struct re_printf *pf);
 void ui_input_str(const char *str);
 int  ui_input_pl(struct re_printf *pf, const struct pl *pl);
-void ui_output(const char *fmt, ...);
-bool ui_isediting(void);
+void ui_output(struct ui_sub *uis, const char *fmt, ...);
+bool ui_isediting(const struct ui_sub *uis);
 int  ui_password_prompt(char **passwordp);
 
 
@@ -763,7 +781,8 @@ struct vidisp_st;
 
 /** Video Display parameters */
 struct vidisp_prm {
-	void *view;  /**< Optional view (set by application or module) */
+	void *view;       /**< Optional view (set by application or module) */
+	bool fullscreen;  /**< Enable fullscreen display                    */
 };
 
 typedef void (vidisp_resize_h)(const struct vidsz *size, void *arg);
@@ -856,8 +875,10 @@ struct videnc_state;
 struct viddec_state;
 struct vidcodec;
 
-typedef int (videnc_packet_h)(bool marker, const uint8_t *hdr, size_t hdr_len,
-			      const uint8_t *pld, size_t pld_len, void *arg);
+typedef int (videnc_packet_h)(bool marker, uint32_t rtp_ts,
+			      const uint8_t *hdr, size_t hdr_len,
+			      const uint8_t *pld, size_t pld_len,
+			      void *arg);
 
 typedef int (videnc_update_h)(struct videnc_state **vesp,
 			      const struct vidcodec *vc,
@@ -971,6 +992,8 @@ int   video_set_source(struct video *v, const char *name, const char *dev);
 void  video_set_devicename(struct video *v, const char *src, const char *disp);
 void  video_encoder_cycle(struct video *video);
 int   video_debug(struct re_printf *pf, const struct video *v);
+uint32_t video_calc_rtp_timestamp(int64_t pts, unsigned fps);
+double video_calc_seconds(uint32_t rtp_ts);
 
 
 /*
@@ -1093,11 +1116,11 @@ int h264_fu_hdr_decode(struct h264_fu *fu, struct mbuf *mb);
 
 const uint8_t *h264_find_startcode(const uint8_t *p, const uint8_t *end);
 
-int h264_packetize(const uint8_t *buf, size_t len, size_t pktsize,
-		   videnc_packet_h *pkth, void *arg);
+int h264_packetize(uint32_t rtp_ts, const uint8_t *buf, size_t len,
+		   size_t pktsize, videnc_packet_h *pkth, void *arg);
 int h264_nal_send(bool first, bool last,
-		  bool marker, uint32_t ihdr, const uint8_t *buf,
-		  size_t size, size_t maxsz,
+		  bool marker, uint32_t ihdr, uint32_t rtp_ts,
+		  const uint8_t *buf, size_t size, size_t maxsz,
 		  videnc_packet_h *pkth, void *arg);
 static inline bool h264_is_keyframe(int type)
 {
@@ -1117,6 +1140,8 @@ static inline bool h264_is_keyframe(int type)
 
 
 int module_preload(const char *module);
+int  module_load(const char *name);
+void module_unload(const char *name);
 
 
 /*
@@ -1148,6 +1173,7 @@ struct list   *baresip_vidcodecl(void);
 struct list   *baresip_vidsrcl(void);
 struct list   *baresip_vidispl(void);
 struct list   *baresip_vidfiltl(void);
+struct ui_sub *baresip_uis(void);
 
 
 #ifdef __cplusplus
