@@ -112,8 +112,8 @@ static int stunsrv_decode(struct account *acc, const struct sip_addr *aor)
 		err |= param_dstr(&acc->stun_pass, &aor->params, "stunpass");
 	else if (pl_isset(&uri.password))
 		err |= pl_strdup(&acc->stun_pass, &uri.password);
-	else
-		err |= pl_strdup(&acc->stun_pass, &aor->uri.password);
+	else if (acc->auth_pass)
+		err |= str_dup(&acc->stun_pass, acc->auth_pass);
 
 	if (pl_isset(&uri.host))
 		err |= pl_strdup(&acc->stun_host, &uri.host);
@@ -386,17 +386,16 @@ int account_alloc(struct account **accp, const char *sipaddr)
 		goto out;
 
 	/* optional password prompt */
-	if (!pl_isset(&acc->laddr.uri.password)) {
-		(void)re_printf("Please enter password for %r@%r: ",
-				&acc->luri.user, &acc->luri.host);
+	if (pl_isset(&acc->laddr.uri.password)) {
 
-		/* TODO: move interactive code away from CORE, to a module */
-		err = ui_password_prompt(&acc->auth_pass);
+		err = re_sdprintf(&acc->auth_pass, "%H",
+				  uri_password_unescape,
+				  &acc->laddr.uri.password);
 		if (err)
 			goto out;
 	}
-	else {
-		err = pl_strdup(&acc->auth_pass, &acc->laddr.uri.password);
+	else if (0 == msg_param_decode(&acc->laddr.params, "auth_pass", &pl)) {
+		err = pl_strdup(&acc->auth_pass, &pl);
 		if (err)
 			goto out;
 	}
@@ -431,6 +430,20 @@ int account_alloc(struct account **accp, const char *sipaddr)
 }
 
 
+int account_set_auth_pass(struct account *acc, const char *pass)
+{
+	if (!acc)
+		return EINVAL;
+
+	acc->auth_pass = mem_deref(acc->auth_pass);
+
+	if (pass)
+		return str_dup(&acc->auth_pass, pass);
+
+	return 0;
+}
+
+
 /**
  * Sets the displayed name. Pass null in dname to disable display name
  *
@@ -518,6 +531,12 @@ enum answermode account_answermode(const struct account *acc)
 }
 
 
+const char *account_aor(const struct account *acc)
+{
+	return acc ? acc->aor : NULL;
+}
+
+
 /**
  * Get the authentication username of an account
  *
@@ -531,6 +550,19 @@ const char *account_auth_user(const struct account *acc)
 }
 
 
+/**
+ * Get the SIP authentication password of an account
+ *
+ * @param acc User-Agent account
+ *
+ * @return Authentication password
+ */
+const char *account_auth_pass(const struct account *acc)
+{
+	return acc ? acc->auth_pass : NULL;
+}
+
+
 /**
  * Get the outbound SIP server of an account
  *
