@@ -29,9 +29,6 @@ static void stream_close(struct stream *strm, int err)
 }
 
 
-/* TODO: should we check RTP per stream, or should we have the
- *       check in struct call instead?
- */
 static void check_rtp_handler(void *arg)
 {
 	struct stream *strm = arg;
@@ -279,6 +276,8 @@ static void rtcp_handler(const struct sa *src, struct rtcp_msg *msg, void *arg)
 	struct stream *s = arg;
 	(void)src;
 
+	s->ts_last = tmr_jiffies();
+
 	if (s->rtcph)
 		s->rtcph(msg, s->arg);
 
