@@ -99,11 +99,13 @@ struct vtx {
 	struct list filtl;                 /**< Filters in encoding order */
 	char device[64];                   /**< Source device name        */
 	int muted_frames;                  /**< # of muted frames sent    */
-	uint32_t ts_tx;                    /**< Outgoing RTP timestamp    */
+	uint32_t ts_offset;                /**< Random timestamp offset   */
 	bool picup;                        /**< Send picture update       */
 	bool muted;                        /**< Muted flag                */
 	int frames;                        /**< Number of frames sent     */
 	int efps;                          /**< Estimated frame-rate      */
+	uint32_t ts_min;
+	uint32_t ts_max;
 };
 
 
@@ -132,14 +134,16 @@ struct vrx {
 	struct lock *lock;                 /**< Lock for decoder          */
 	struct list filtl;                 /**< Filters in decoding order */
 	struct tmr tmr_picup;              /**< Picture update timer      */
+	struct vidsz size;                 /**< Incoming video resolution */
 	enum vidorient orient;             /**< Display orientation       */
 	char device[64];                   /**< Display device name       */
-	bool fullscreen;                   /**< Fullscreen flag           */
 	int pt_rx;                         /**< Incoming RTP payload type */
 	int frames;                        /**< Number of frames received */
 	int efps;                          /**< Estimated frame-rate      */
 	unsigned n_intra;                  /**< Intra-frames decoded      */
 	unsigned n_picup;                  /**< Picture updates sent      */
+	uint32_t ts_min;
+	uint32_t ts_max;
 };
 
 
@@ -337,15 +341,27 @@ static int get_fps(const struct video *v)
 }
 
 
-static int packet_handler(bool marker, const uint8_t *hdr, size_t hdr_len,
-			  const uint8_t *pld, size_t pld_len, void *arg)
+static int packet_handler(bool marker, uint32_t ts,
+			  const uint8_t *hdr, size_t hdr_len,
+			  const uint8_t *pld, size_t pld_len,
+			  void *arg)
 {
 	struct vtx *vtx = arg;
 	struct stream *strm = vtx->video->strm;
 	struct vidqent *qent;
+	uint32_t rtp_ts;
 	int err;
 
-	err = vidqent_alloc(&qent, marker, strm->pt_enc, vtx->ts_tx,
+	/* NOTE: does not handle timestamp wrap around */
+	if (ts < vtx->ts_min)
+		vtx->ts_min = ts;
+	if (ts > vtx->ts_max)
+		vtx->ts_max = ts;
+
+	/* add random timestamp offset */
+	rtp_ts = vtx->ts_offset + ts;
+
+	err = vidqent_alloc(&qent, marker, strm->pt_enc, rtp_ts,
 			    hdr, hdr_len, pld, pld_len);
 	if (err)
 		return err;
@@ -424,7 +440,6 @@ static void encode_rtp_send(struct vtx *vtx, struct vidframe *frame)
 	if (err)
 		return;
 
-	vtx->ts_tx += (SRATE/vtx->vsrc_prm.fps);
 	vtx->picup = false;
 }
 
@@ -478,12 +493,16 @@ static int vtx_alloc(struct vtx *vtx, struct video *video)
 	tmr_init(&vtx->tmr_rtp);
 
 	vtx->video = video;
-	vtx->ts_tx = 160;
+
+	/* The initial value of the timestamp SHOULD be random */
+	vtx->ts_offset = rand_u16();
 
 	str_ncpy(vtx->device, video->cfg.src_dev, sizeof(vtx->device));
 
 	tmr_start(&vtx->tmr_rtp, 1, rtp_tmr_handler, vtx);
 
+	vtx->ts_min = ~0;
+
 	return err;
 }
 
@@ -502,6 +521,8 @@ static int vrx_alloc(struct vrx *vrx, struct video *video)
 
 	str_ncpy(vrx->device, video->cfg.disp_dev, sizeof(vrx->device));
 
+	vrx->ts_min = ~0;
+
 	return err;
 }
 
@@ -564,6 +585,13 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 		goto out;
 	}
 
+	/* todo: check if RTP timestamp wraps */
+
+	if (hdr->ts < vrx->ts_min)
+		vrx->ts_min = hdr->ts;
+	if (hdr->ts > vrx->ts_max)
+		vrx->ts_max = hdr->ts;
+
 	frame->data[0] = NULL;
 	err = vrx->vc->dech(vrx->dec, frame, &intra, hdr->m, hdr->seq, mb);
 	if (err) {
@@ -589,6 +617,8 @@ static int video_stream_decode(struct vrx *vrx, const struct rtp_header *hdr,
 	if (!vidframe_isvalid(frame))
 		goto out;
 
+	vrx->size = frame->size;
+
 	if (!list_isempty(&vrx->filtl)) {
 
 		err = vidframe_alloc(&frame_filt, frame->fmt, &frame->size);
@@ -871,6 +901,7 @@ static int set_vidisp(struct vrx *vrx)
 
 	vrx->vidisp = mem_deref(vrx->vidisp);
 	vrx->vidisp_prm.view = NULL;
+	vrx->vidisp_prm.fullscreen = vrx->video->cfg.fullscreen;
 
 	vd = (struct vidisp *)vidisp_find(baresip_vidispl(),
 					  vrx->video->cfg.disp_mod);
@@ -1038,7 +1069,7 @@ static int vidisp_update(struct vrx *vrx)
 	int err = 0;
 
 	if (vd->updateh) {
-		err = vd->updateh(vrx->vidisp, vrx->fullscreen,
+		err = vd->updateh(vrx->vidisp, vrx->vidisp_prm.fullscreen,
 				  vrx->orient, NULL);
 	}
 
@@ -1059,7 +1090,7 @@ int video_set_fullscreen(struct video *v, bool fs)
 	if (!v)
 		return EINVAL;
 
-	v->vrx.fullscreen = fs;
+	v->vrx.vidisp_prm.fullscreen = fs;
 
 	return vidisp_update(&v->vrx);
 }
@@ -1287,13 +1318,21 @@ int video_debug(struct re_printf *pf, const struct video *v)
 
 	err = re_hprintf(pf, "\n--- Video stream ---\n");
 	err |= re_hprintf(pf, " started: %s\n", v->started ? "yes" : "no");
+
 	err |= re_hprintf(pf, " tx: %u x %u, fps=%d\n",
 			  vtx->vsrc_size.w,
 			  vtx->vsrc_size.h, vtx->vsrc_prm.fps);
 	err |= re_hprintf(pf, "     skipc=%u\n", vtx->skipc);
-	err |= re_hprintf(pf, " rx: pt=%d\n", vrx->pt_rx);
+	err |= re_hprintf(pf, "     time = %.3f sec\n",
+			  video_calc_seconds(vtx->ts_max - vtx->ts_min));
+
+	err |= re_hprintf(pf, " rx: %u x %u\n", vrx->size.w, vrx->size.h);
+	err |= re_hprintf(pf, "     pt=%d\n", vrx->pt_rx);
+
 	err |= re_hprintf(pf, "     n_intra=%u, n_picup=%u\n",
 			  vrx->n_intra, vrx->n_picup);
+	err |= re_hprintf(pf, "     time = %.3f sec\n",
+			  video_calc_seconds(vrx->ts_max - vrx->ts_min));
 
 	if (!list_isempty(baresip_vidfiltl())) {
 		err |= vtx_print_pipeline(pf, vtx);
@@ -1345,3 +1384,38 @@ void video_set_devicename(struct video *v, const char *src, const char *disp)
 	str_ncpy(v->vtx.device, src, sizeof(v->vtx.device));
 	str_ncpy(v->vrx.device, disp, sizeof(v->vrx.device));
 }
+
+
+/**
+ * Calculate the RTP timestamp from Presentation Time Stamp (PTS)
+ * or Decoding Time Stamp (DTS) and framerate.
+ *
+ * @note The calculated RTP Timestamp may wrap around.
+ *
+ * @param pts Presentation Time Stamp (PTS)
+ * @param fps Framerate in [frames per second]
+ *
+ * @return RTP Timestamp
+ */
+uint32_t video_calc_rtp_timestamp(int64_t pts, unsigned fps)
+{
+       uint64_t rtp_ts;
+
+       if (!fps)
+	       return 0;
+
+       rtp_ts = ((uint64_t)SRATE * pts) / fps;
+
+       return (uint32_t)rtp_ts;
+}
+
+
+double video_calc_seconds(uint32_t rtp_ts)
+{
+	double timestamp;
+
+	/* convert from RTP clockrate to seconds */
+	timestamp = (double)rtp_ts / (double)SRATE;
+
+	return timestamp;
+}
